#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "xml_trace_sense4xi.py">
#<abstract4xi abstract = "senses tracing and produces XML"/>
#<refer4xi reference = "yourDragonXi"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
#<note4xi note = "Sensored data"/>   
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'xml_trace_sense4xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    from time import localtime, strftime         
except:
    pass
        
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    pass
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from print4xi import Print4Xi
from print4xi import print4xi
from time4xi import time4xi
#"
#/>

        
# -------------------------------------------------------------------------------------------------
# xml_trace_sense4xi                            
#
# usage 1 (see function print_trace4xi of test_cpu_corecount4xi.py)
#    context2sense = {
#     'file_path4xi': '..\\..\\xml\\sense-test-cpu-corecount4xi.xml',
#     'name_space4xi': '   xmlns="http://www.yourdragonxi.org/sense/sw4xi/python32/owl/multiprocessing4xi.owl#"\n',
#     'xml_base4xi': '   xml:base="http://www.yourdragonxi.org.com/sense/sw4xi/python32/owl/multiprocessing4xi.owl">\n',
#     'ontology_about4xi': '   <owl:Ontology rdf:about="Multiprocessing4Xi">\n',
#     'ontology_comment4xi': '      <rdfs:comment xml:lang="en">Dragonized multiprocessing for multi-core systems</rdfs:comment>\n',
#     'class0_id4xi': '   <owl:Class rdf:ID="Dragon4Xi#0">\n',
#     'class0_comment4xi': '      <rdfs:comment xml:lang="en">testing of cpu corecount in system</rdfs:comment>\n',
#     'object_property_rdf4xi': '   <owl:ObjectProperty rdf:ID="test-cpu-corecount#0">\n',
#     'object_property_comment4xi': '       <rdfs:comment xml:lang="en">sensed messages of active classes, methods and functions</rdfs:comment>\n'
#     'debug4xi': True}
#      
#    xml_trace_sense4xi(t2xi, context2sense)     <-- t2xi is pointer to Print4Xi object, xml file tied to context will be generated
#
# usage 2:
#    xml_trace_sense4xi(t2xi)                    <-- xml file with default values will be generated
#
# usage 3:
#    xml_trace_sense4xi(t2xi,{})                 <-- xml file with default values will be generated
#
# usage 4:
#    xml_trace_sense4xi(t2xi,{'debug4xi': True}) <-- xml file with default values will be generated plus trace from print 
# 
# -------------------------------------------------------------------------------------------------    
def xml_trace_sense4xi (xml_trace_sense4xi_param1, xml_trace_sense4xi_param2 = 
    {'file_path4xi': '..\\..\\xml\\sense-default-trace4xi.xml',
     'name_space4xi': '   xmlns="http://www.dragonxi2.com/sense/sw4xi/python32/xml/multiprocessing4xi.owl#"\n',
     'xml_base4xi': '   xml:base="http://www.dragonxi2.com/sense/sw4xi/python32/xml/multiprocessing4xi.owl">\n',
     'ontology_about4xi': '   <owl:Ontology rdf:about="DragonXi Multiprocessing">\n',
     'ontology_comment4xi': '      <rdfs:comment xml:lang="en">sensed classes, methods, functions and messages</rdfs:comment>\n',
     'class0_id4xi': '   <owl:Class rdf:ID="Dragon4Xi#000000">\n',
     'class0_comment4xi': '      <rdfs:comment xml:lang="en">dragonized messages for tracing</rdfs:comment>\n',
     'object_property_rdf4xi': '   <owl:ObjectProperty rdf:ID="sensing#000000">\n',
     'object_property_comment4xi': '       <rdfs:comment xml:lang="en">sense</rdfs:comment>\n',
     'debug4xi': True}):
    
    
    '''
    xml_trace_sense4xi senses traced knowledgebase producing XML
    
    '''       
    # ---------------------------------------------------------------------------------------------
    # STARTING                         
    # ---------------------------------------------------------------------------------------------     
    try:
        t2xi = xml_trace_sense4xi_param1 # pointer to Print4Xi object, mandatory
        c2xi = xml_trace_sense4xi_param2 # context, optional
        keys2context = c2xi.keys()
    except:
        pass
        
    # --------------------------------------------------------------------------------------------- 
    # TAILORING 
    # ---------------------------------------------------------------------------------------------     
    if 'debug4xi' in keys2context:
        debug2xi = c2xi['debug4xi']     
    else:
        debug2xi = False
  
    # ---------------------------------------------------------------------------------------------
    # SENSE
    # ---------------------------------------------------------------------------------------------    
    if debug2xi is True:
        print(33*("=")) 
        print("(STEP 1A/33) OF XML_TRACE_SENSE - TRYING TO WRITE SENSED KNOWLEDGEBASE")    
        print(33*("="))
        print("")
        print("(STEP 1B/33) OF XML_TRACE_SENSE - SET T2XI TO: ", t2xi)
        # <print4xi.Print4Xi object at 0x0000000003828AC8>     
        print("")
        print("(STEP 1B/33) OF XML_TRACE_SENSE - SET C2XI TO: ", c2xi)
        print("")
    try:            
        kb2xi = t2xi.sense() 
        len2kb = len(kb2xi)
        if debug2xi is True:
            print("(STEP 2A/33) OF XML_TRACE_SENSE - LEN2KB: ", len2kb)
            # 3191
        keys2kb = kb2xi.keys()
        if debug2xi is True:
            for key2kb in sorted(keys2kb):
                print("(STEP 3A/33) OF XML_TRACE_SENSE - KEY2KB: ", key2kb)
                # <class4xi>BoundedSemaphore4Xi</class4xi><method4xi>__init__</method4xi><order4xi>#000000391</order4xi>
                # ...
                # <class4xi>Pipe4Xi</class4xi><method4xi>Pipe4Xi</method4xi><order4xi>#000000068</order4xi>
                # <class4xi>Queue4Xi</class4xi><method4xi>__init__</method4xi><order4xi>#000000390</order4xi>
                # <class4xi>fetch2scope4xi</class4xi><method4xi>fetch2scope4xi</method4xi><order4xi>#000001059</order4xi>
                # ...
            for key2kb in sorted(keys2kb):
                print("(STEP 3B/33) OF XML_TRACE_SENSE - VALUE: ", kb2xi[key2kb])                  
                # <message4xi>{(Step 1/7b) started; init-of-BoundedSemaphore4Xi-started: 06:02:42 10/17/11 Pacific Daylight Time}</message4xi>
                # <message4xi>{(Step 0A/33) using; t2xi-using-by-fetch2scope4xi: print4xi.Print4Xi object at 0x0000000003828AC8; time: 06:02:42 10/17/11 Pacific Daylight Time}</message4xi>                      
                # ...
    except:
        print("xml_trace_sense4xi *** failed with knowledgebase from the sense method of Print4Xi: ", time4xi())

    
    # ---------------------------------------------------------------------------------------------
    # OPEN 
    # ---------------------------------------------------------------------------------------------     
    try:
        if 'file_path4xi' in keys2context: 
            filepath2xi = c2xi['file_path4xi']
        else:
            filepath2xi = '..\\..\\xml\\sense-test-default4xi.xml' 
        if debug2xi is True:
            print("xml_trace_sense4xi: filepath2xi: ", filepath2xi)
            print("")        
        f2xi=open(filepath2xi, 'w')
        if debug2xi is True:
            print("xml_trace_sense4xi: opened f2xi: ", f2xi)
            print("")    
    except:
        print("xml_trace_sense4xi *** failed to open - filepath2xi: ", filepath2xi)
        
    # ---------------------------------------------------------------------------------------------
    # WRITE HEADERS
    # --------------------------------------------------------------------------------------------- 
    try:
        if debug2xi is True:
            print(33*("=")) 
            print("xml_trace_sense4xi: opened XML file: ", filepath2xi)          
            print(33*("=")) 
            
            print(33*("-"))  
            print("xml_trace_sense4xi: create header for XML file")     
            print(33*("-"))    
        
          
        f2xi.write('<?xml version="1.0"?>\n')
        f2xi.write('<rdf:RDF\n')
        f2xi.write('   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n')
        f2xi.write('   xmlns:xsd="http://www.w3.org/2001/XMLSchema#"\n')
        f2xi.write('   xmlns:owl="http://www.w3.org/2002/07/owl#"\n')
        f2xi.write('   xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"\n')
    except:
        print("xml_trace_sense4xi *** failed to write headers ***")
        
    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE 
    # ---------------------------------------------------------------------------------------------     
    try:
        if 'name_space4xi' in keys2context:
            namespace2xi = c2xi['name_space4xi'] 
        else:
            namespace2xi = '   xmlns="http://www.yourdragonxi.org/web/lib/owl/Default4Xi.owl#"\n'
               
        f2xi.write(namespace2xi)

    except:
        print("xml_trace_sense4xi *** failed with namespace ***")
    # ---------------------------------------------------------------------------------------------
    # WRITE 
    # ---------------------------------------------------------------------------------------------     
    try:

        f2xi.write('   xmlns:p1="http://www.owl-ontologies.com/assert.owl#"\n')    
    except:
        print("xml_trace_sense4xi *** failed to write namespace ***")             
    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE 
    # ---------------------------------------------------------------------------------------------      
    try:
        if 'xml_base4xi' in keys2context: 
            xmlbase2xi = c2xi['xml_base4xi'] 
        else:
            xmlbase2xi = '   xml:base="http://www.yourdragonxi.org.com/web/lib/owl/sense-default4xi.owl">\n'
        f2xi.write(xmlbase2xi)
    except:
        print("xml_trace_sense4xi *** failed to write xmlbase ***")        

    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE  
    # ---------------------------------------------------------------------------------------------       
    try:
        if 'ontology_about4xi' in keys2context: 
            ontology_about2xi = c2xi['ontology_about4xi']
        else:
            ontology_about2xi = '   <owl:Ontology rdf:about="">\n' 
        f2xi.write(ontology_about2xi)
    except:
        print("xml_trace_sense4xi *** failed to write ontology about ***")      
 
    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE  
    # ---------------------------------------------------------------------------------------------       
    try:
        if 'ontology_comment4xi' in keys2context:
            ontology_comment2xi = c2xi['ontology_comment4xi']
        else:
            ontology_comment2xi = '      <rdfs:comment xml:lang="en">sense-default4xi</rdfs:comment>\n'    
        f2xi.write(ontology_comment2xi)
      
        f2xi.write('      <owl:versionInfo xml:lang="en">Version ' + strftime("%a, %d %b %Y %H:%M:%S",localtime()) + '</owl:versionInfo>\n')
    except:
        print("xml_trace_sense4xi *** failed to write ontology comment ***")   
            
    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE  
    # ---------------------------------------------------------------------------------------------       
    try:
        if 'class0_id4xi' in keys2context:
            class2xi = c2xi['class0_id4xi']
        else:
            class2xi = '   <owl:Class rdf:ID="#0">\n'    
        f2xi.write(class2xi)
    except:
        print("xml_trace_sense4xi *** failed to write Class ID ***") 
        
    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE  
    # ---------------------------------------------------------------------------------------------           
    try:
        if 'class0_comment4xi' in keys2context:
            class_comment2xi = c2xi['class0_comment4xi']
        else:
            class_comment2xi = '      <rdfs:comment xml:lang="en">Default Messages</rdfs:comment>\n'
        f2xi.write(class_comment2xi)

    except:
        print("xml_trace_sense4xi *** failed to write class_comment ***")  
    # ---------------------------------------------------------------------------------------------
    # WRITE 
    # ---------------------------------------------------------------------------------------------     
    try:
        f2xi.write('   </owl:Class>\n')   
    except:
        pass         
    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE  
    # ---------------------------------------------------------------------------------------------       
    try:
        if 'object_property_rdf4xi' in keys2context:
            object_property2xi = c2xi['object_property_rdf4xi']
        else:
            object_property2xi = '   <owl:ObjectProperty rdf:ID="sensing#0">\n'    
        f2xi.write(object_property2xi)
    except:
        print("xml_trace_sense4xi *** failed to write object_property ***")  
      
    # ---------------------------------------------------------------------------------------------
    # TAILORED WRITE  
    # ---------------------------------------------------------------------------------------------       
    try:
        if 'object_property_comment4xi' in keys2context:
            object_property_comment2xi = c2xi['object_property_comment4xi']
        else:
            object_property_comment2xi = '       <rdfs:comment xml:lang="en">sense</rdfs:comment>\n'        
        f2xi.write(object_property_comment2xi)
    except:
        print("xml_trace_sense4xi *** failed to object property comment ***")  
                 
    # ---------------------------------------------------------------------------------------------
    # WRITE  
    # --------------------------------------------------------------------------------------------- 
    try:      
        f2xi.write('   </owl:ObjectProperty>\n')
    except:
        print("xml_trace_sense4xi *** failed to to write end of ObjectProperty ***")  
         
    # ---------------------------------------------------------------------------------------------
    # TAILOR 
    # ---------------------------------------------------------------------------------------------            
    if debug2xi is True:
        print(33*("-"))
        print("SENSING of xml_trace_sense4xi: create body for XML file")     
        print(33*("-"))
          
    orderIRI2xi = "#0"
    classIRI2xi = "to-be-solved-class"
    methodIRI2xi = "to-be-solved-method"
        
    classPropertyID2xi = 1
    classComment2xi = "Comment of Class "
    objectPropertID2Xi = 1
    objectPropertComment2xi = "Comment of Object Property "     
    
    
    # ---------------------------------------------------------------------------------------------
    #  
    # ---------------------------------------------------------------------------------------------
    try:    
        for key2kb in sorted(keys2kb):
            # -----------------------------------------------------------------------------------------
            #  
            # -----------------------------------------------------------------------------------------           
             
            while 1:
                try:                                                                              
                    start_index2order = str.find(key2kb,'<order4xi>')               # 
                    end_index2order = str.find(key2kb,'</order4xi>')
                         
                    if (end_index2order == -1) or (start_index2order == -1):
                        print("*** error: did not find order number inside the key of sensed dictionary ***!")   
                        break
                    else:
                        orderIRI2xi = key2kb[start_index2order+10:end_index2order]  #
                        break    
                except:
                    break 
            # -----------------------------------------------------------------------------------------
            #  
            # -----------------------------------------------------------------------------------------           
                                          
            while 1:
                try:                                                                              
                    start_index2class = str.find(key2kb,'<class4xi>')                # 
                    end_index2class = str.find(key2kb,'</class4xi>')
                         
                    if (end_index2class == -1) or (start_index2class == -1):
                        print("*** error: did not find class4xi tag inside the key of sensed dictionary ***!")   
                        break
                    else:
                        classIRI2xi = key2kb[start_index2class+10:end_index2class]  # 
                        break 
                except:
                    break
            # -----------------------------------------------------------------------------------------
            #  
            # -----------------------------------------------------------------------------------------           
                                                                           
            while 1:
                try:                                                                              
                    start_index2method = str.find(key2kb,'<method4xi>')              # 
                    end_index2method = str.find(key2kb,'</method4xi>')
                         
                    if (end_index2method == -1) or (start_index2method == -1):
                        print("*** error: did not find class4xi tag inside the key of sensed dictionary ***!")   
                        break
                    else:
                        methodIRI2xi = key2kb[start_index2method+11:end_index2method] # 
                        break       
                except:
                    break
            try:        
                f2xi.write('   <owl:Class rdf:ID="' + classIRI2xi + orderIRI2xi + '">\n')           # 
            except:
                pass
                
               
            # -----------------------------------------------------------------------------------------
            # SENSING: MOVED FILTERING < and > TAGS IN MESSAGES TO PRINT4XI.PY 
            # -----------------------------------------------------------------------------------------        
            try:
                msgIRI2xi = kb2xi[key2kb]                                         
                                       
                if debug2xi is True:
                    print("SENSE writing: ", classIRI2xi + methodIRI2xi + msgIRI2xi)             
                
                if classIRI2xi == methodIRI2xi:
                    f2xi.write('      <rdfs:comment xml:lang="en">' + '<function4xi>' + methodIRI2xi + '</function4xi>' + msgIRI2xi + '</rdfs:comment>\n')         
                else:
                    f2xi.write('      <rdfs:comment xml:lang="en">' + '<method4xi>' + methodIRI2xi + '</method4xi>' + msgIRI2xi + '</rdfs:comment>\n') 
                
                f2xi.write('   </owl:Class>\n')
            except:
                pass
    except:
        print("xml_trace_sense4xi *** failed with kb ***")     

    # ---------------------------------------------------------------------------------------------
    # END OF ONTOLOGY, RDF AND COPYRIGHT                         
    # ---------------------------------------------------------------------------------------------                   
    if debug2xi is True:
        print(33*("-"))
        print("SENSING of xml_trace_sense4xi: create end for XML file")     
        print(33*("-"))    
    try:            
        f2xi.write('   </owl:Ontology>\n')
        f2xi.write('</rdf:RDF>\n')
        f2xi.write('<!-- Copyright Small & Smart Inc. All rights reserved. -->\n')
    except:
        print("xml_trace_sense4xi *** failed with end of ontology ***")
         
    # ---------------------------------------------------------------------------------------------
    # CLOSE                         
    # --------------------------------------------------------------------------------------------- 
    if debug2xi is True:
        print(33*("-"))
        print("xml_trace_sense4xi: trying to close XML file: ", filepath2xi)     
        print(33*("-"))
        
    try:                  
        f2xi.close()
        if debug2xi is True:
            print(33*("-"))
            print("xml_trace_sense4xi: closed XML file: ", filepath2xi)     
            print(33*("-"))    
    except:
        print("xml_trace_sense4xi ** failed to close XML file: ", filepath2xi)
        
    # ---------------------------------------------------------------------------------------------
    # RETURN                         
    # ---------------------------------------------------------------------------------------------    
    if debug2xi is True:
        print(33*("-"))
        print("xml_trace_sense4xi: returning ", time4xi())     
        print(33*("-"))
        

#</program4xi>
         
# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   sense*4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of xml_trace_sense4xi.py                          
# ------------------------------------------------------------------------------------------------- 
#"
#/>            

